#!/usr/bin/env python
#
# This file is part of Maze Template
# Author(s): Clare Richardson, Girlstart
# Copyright 2008
#
# Source code is part of Girlstart's Project IT Girl Game Templates:
# http://itgirl.dreamhosters.com/itgirlgames/, http://www.girlstart.org/itgirl
# See the credits file for font, sound, and image sources.
#
# Source code is derived from Pyman:
# http://www.learningpython.com/2006/03/12/creating-a-game-in-python-using-pygame-part-one/
#
# Maze Template is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Maze Template is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Maze Template.  If not, see <http://www.gnu.org/licenses/>.

import pygame, random
from helpers import *

class Player(pygame.sprite.Sprite):
	# This is our player that will move around the screen controlled by the user
	
	def __init__(self, centerPoint, image = None):
		pygame.sprite.Sprite.__init__(self) 
		
		if image == None:
			self.image, self.rect = load_image('pacman.png',-1)
		else:
			self.image = image
			self.rect = image.get_rect()
		self.rect.center = centerPoint

		# Initialize the number of pellets eaten
		self.pellets = 0
		
		# Set the number of Pixels to move each time
		self.x_dist = 10
		self.y_dist = 10 

	def update(self,key,block_group):
		xMove,yMove = 0,0

		if (key == K_RIGHT):
			xMove += self.x_dist
		elif (key == K_LEFT):
			xMove += -self.x_dist
		elif (key == K_UP):
			yMove += -self.y_dist
		elif (key == K_DOWN):
			yMove += self.y_dist

		self.rect.move_ip(xMove,yMove)

		# If we hit a block, don't move - reverse the movement
		if pygame.sprite.spritecollide(self, block_group, False):
			self.rect.move_ip(-xMove,-yMove)

	def dead_end(self,block_group):
		allowed_moves = 4

		# try moving right
		self.rect.move_ip(self.x_dist,0)
		if pygame.sprite.spritecollide(self, block_group, False): # blocked by a wall
			allowed_moves -= 1
		self.rect.move_ip(-self.x_dist,0)

		# try moving left
		self.rect.move_ip(-self.x_dist,0)
		if pygame.sprite.spritecollide(self, block_group, False): # blocked by a wall
			allowed_moves -= 1
		self.rect.move_ip(self.x_dist,0)

		# try moving up
		self.rect.move_ip(0,-self.y_dist)
		if pygame.sprite.spritecollide(self, block_group, False): # blocked by a wall
			allowed_moves -= 1
		self.rect.move_ip(0,self.y_dist)

		# try moving down
		self.rect.move_ip(0,self.y_dist)
		if pygame.sprite.spritecollide(self, block_group, False): # blocked by a wall
			allowed_moves -= 1
		self.rect.move_ip(0,-self.y_dist)

		if allowed_moves <= 1: # blocked by a wall in at least 3 directions
			return True
		else:
			return False

class Monster(pygame.sprite.Sprite):
	# This is our Monster that will move around the screen randomly
	
	def __init__(self, centerPoint, image = None):
		pygame.sprite.Sprite.__init__(self) 
		
		if image == None:
			self.image, self.rect = load_image('monster.png',-1)
		else:
			self.image = image
			self.rect = image.get_rect()
		self.rect.center = centerPoint

		# Initialize the direction and speed:
		self.direction = random.randint(1,4)
		self.speed = 1  # how many pixels to move each time
		self.moves = random.randint(50,100)
		self.moveCount = 0
			
	def update(self,block_group,block_group2=None):
		xMove,yMove = 0,0
		
		if self.direction==1:
			xMove = -self.speed
		elif self.direction==2:
			yMove = -self.speed
		elif self.direction==3:
			xMove = self.speed
		elif self.direction==4:
			yMove = self.speed
		
		self.rect.move_ip(xMove,yMove)
		self.moveCount += 1
		
		if pygame.sprite.spritecollideany(self, block_group) or pygame.sprite.spritecollideany(self, block_group2):
			# If we hit a block or finish line, don't move - reverse the movement
			self.rect.move_ip(-xMove,-yMove)
			self.direction = random.randint(1,4)
		elif self.moves == self.moveCount:
			# If we have moved enough, choose a new direction
			self.direction = random.randint(1,4)
			self.moves = random.randint(50,100)
			self.moveCount = 0

class Pellet(pygame.sprite.Sprite):
		
	def __init__(self, centerPoint, image = None):
		pygame.sprite.Sprite.__init__(self) 
		
		if image == None:
			self.image, self.rect = load_image('pellet.png',-1)
		else:
			self.image = image
			self.rect = image.get_rect()
		self.rect.center = centerPoint

class Block(pygame.sprite.Sprite):
		
	def __init__(self, centerPoint, image = None):
		pygame.sprite.Sprite.__init__(self) 
		
		if image == None:
			self.image, self.rect = load_image('block.png',-1)
		else:
			self.image = image
			self.rect = image.get_rect()
		self.rect.center = centerPoint

class Space(pygame.sprite.Sprite):
		
	def __init__(self, centerPoint, image = None):
		pygame.sprite.Sprite.__init__(self) 
		
		if image == None:
			self.image, self.rect = load_image('space.png',-1)
		else:
			self.image = image
			self.rect = image.get_rect()
		self.rect.center = centerPoint

class FinishLine(pygame.sprite.Sprite):
		
	def __init__(self, centerPoint, image = None):
		pygame.sprite.Sprite.__init__(self) 
		
		if image == None:
			self.image, self.rect = load_image('finish.png')
		else:
			self.image = image
			self.rect = image.get_rect()
		self.rect.center = centerPoint
